package it.uniroma3.plasm.editors;

import it.uniroma3.plasm.editors.util.PlasmColorProvider;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextDoubleClickStrategy;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.BufferedRuleBasedScanner;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;

/**
 * This class bundles the whole configuration space of a source viewer.
 * Instances of this class are passed to the configure method of ISourceViewer.
 * 
 * @author EcT(o)PLaSM Group
 */
public class PlasmSourceViewerConfiguration extends SourceViewerConfiguration {

	/** The double clicks strategy to adopt in the PLaSM editor */
	private PlasmDoubleClickStrategy doubleClickStrategy;

	/**
	 * The code scanner takes care of scanning the PLaSM code 
	 * and identifying different tokens
	 */
	private PlasmCodeScanner codeScanner;

	/** Divides the PLaSM file into differents environments: comments or code */
	private PlasmPartitionScanner partitionScanner;

	/** Provides all differents colors used in syntax highlighting */
	private PlasmColorProvider colorProvider;

	/**
	 * Default constructor: it only associate the internal color provider
	 * with a PLaSM color provider
	 * 
	 * @param colorProvider a color provider with PLaSM colors
	 */
	public PlasmSourceViewerConfiguration(PlasmColorProvider colorProvider) {
		this.colorProvider = colorProvider;
	}

	/**
	 * Default getter for the ConfiguredContentTypes from sourceViewer
	 * 
	 * @param sourceViewer the SourceViewer associated with current editor
	 * @return the different PLaSM partition types (comments or code)
	 */
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return PlasmPartitionScanner.PSM_PARTITION_TYPES;
	}

	/**
	 * Default getter for the doubleClickStyrategy associated with PLaSM
	 * environment
	 * 
	 * @return the doubleClickStrategy object in use
	 */
	public ITextDoubleClickStrategy getDoubleClickStrategy(
			ISourceViewer sourceViewer, String contentType) {
		if (doubleClickStrategy == null)
			doubleClickStrategy = new PlasmDoubleClickStrategy();
		return doubleClickStrategy;
	}
	
	/**
	 * Default getter for PlasmCodeScanner
	 * 
	 * @return the PlasmCodeScanner in use
	 */
	protected PlasmCodeScanner getPlasmCodeScanner() {
		if (this.codeScanner == null) {
			codeScanner = new PlasmCodeScanner(colorProvider);
			codeScanner.setDefaultReturnToken(new Token(new TextAttribute(
					colorProvider.getColor(PlasmColorProvider.DEFAULT))));
		}
		return codeScanner;
	}

	/**
	 * Standard getter for PlasmPartitionScanner
	 * 
	 * @return PlasmPartitionScanner
	 */
	protected PlasmPartitionScanner getPlasmPartitionScanner() {
		if (partitionScanner == null)
			partitionScanner = new PlasmPartitionScanner();

		return partitionScanner;
	}

	/**
	 * Define the Presentation Reconciler. A Reconceiler monitors the changes to
	 * the text through its associated viewer. It uses the document's regions to
	 * determine the content types affected by the change and notifies a damager
	 * that is appropriate for the affected content type. Once the damage is
	 * computed, it is passed to the appropriate repairer which will construct
	 * repair descriptions that are applied to the viewer to put it back in sync
	 * with the underlying content.
	 */
	public IPresentationReconciler getPresentationReconciler(
			ISourceViewer sourceViewer) {

		PresentationReconciler reconciler = new PresentationReconciler();
		DefaultDamagerRepairer dr;
		NonRuleBasedDamagerRepairer ndr;

		reconciler
				.setDocumentPartitioning(getConfiguredDocumentPartitioning(sourceViewer));

		dr = new DefaultDamagerRepairer(getPlasmCodeScanner());
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

		dr = new DefaultDamagerRepairer(new SingleTokenScanner(
				new TextAttribute(colorProvider
						.getColor(PlasmColorProvider.COMMENT))));
		reconciler.setDamager(dr, PlasmPartitionScanner.COMMENT);
		reconciler.setRepairer(dr, PlasmPartitionScanner.COMMENT);

		return reconciler;
	}

	/**
	 * The single token scanner class.
	 */
	static class SingleTokenScanner extends BufferedRuleBasedScanner {

		public SingleTokenScanner(TextAttribute attribute) {
			setDefaultReturnToken(new Token(attribute));
		}
	}

}