package it.uniroma3.plasm.editors;

import it.uniroma3.plasm.editors.util.PlasmColorProvider;
import it.uniroma3.plasm.editors.util.PlasmOperatorDetector;
import it.uniroma3.plasm.editors.util.PlasmWhitespaceDetector;
import it.uniroma3.plasm.editors.util.PlasmWordDetector;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.NumberRule;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;

/**
 * A rule based PLaSM code scanner. It contains all rules to classify codewords
 * between Keywords, Digits, Strings, Comments, ... Add your code rules here.
 * 
 * @author EcT(o)PLaSM Group
 */
public class PlasmCodeScanner extends RuleBasedScanner {

	/** This array contains all PLaSM keywords */
	private static String[] keywords = {"aa", "abs", "ac", "acolor", "acos",
			"actor", "al", "alias", "align", "and", "animation", "animation",
			"appearance", "apply", "ar", "arc", "as", "asin", "assoc", "atan",
			"axialcamera", "axialcameras", "basehermite", "bbox", "bbox",
			"bernstein", "bernsteinbasis", "bezier", "beziercurve",
			"beziermanifold", "bezierstripe", "beziersurface", "bigger",
			"biggest", "bilinearsurface", "binormal", "biquadraticsurface",
			"black", "blend", "blue", "bottom", "box", "brown", "bspize",
			"bspline", "bsplinebasis", "c", "cabinet", "camera", "cart",
			"case", "cat", "catch", "ceil", "centeredcameras",
			"centralcavalier", "char", "charseq", "choose", "circle",
			"circumference", "class", "cmap", "color", "comp", "computecoords",
			"cone", "conicalsurface", "cons", "control", "convexcoords",
			"convexhull", "coonspatch", "copy", "cos", "cosh", "crease",
			"crosspolytope", "cube", "cubiccardinal", "cubiccardinalbasis",
			"cubichermite", "cubicubspline", "cubicubsplinebasis", "cuboid",
			"curl", "curvature", "curve2cspath", "curve2mapvect", "cyan",
			"cylinder", "cylindricalsurface", "d", "deboor", "def", "depol",
			"depth_sort", "depth_test", "derbernstein", "derbernsteinbase",
			"derbezier", "determinant", "difference", "differencepr", "dim",
			"dimetric", "dirproject", "displaygraph", "displaynubspline",
			"displaynurbspline", "distl", "distr", "div", "divergence",
			"dodecahedron", "dot", "down", "dp", "drawedges", "drawforks",
			"drawtree", "ds", "dsphere", "dump", "dumprep", "ellipse", "embed",
			"end", "eq", "ex", "exp", "explode", "export", "extract_bodies",
			"extract_polygons", "extract_wires", "extrude", "extrusion",
			"fact", "false", "feature", "ff", "fillcolor", "filter",
			"finitecone", "first", "flash", "flashani", "floor", "fontcolor",
			"fontheight", "fontspacing", "fontwidth", "fractalsimplex",
			"frame", "frame", "frameflash", "fromto", "gausscurvature", "ge",
			"grad", "gradient", "gradmap", "gray", "green", "gt", "help",
			"hermite", "hermitebasis", "hermitesurface", "hexahedron",
			"icosahedron", "id", "idnt", "if", "in", "inarcs", "innerprod",
			"inset", "insl", "insr", "intersection", "intersectionpr",
			"intervals", "intmax", "intmin", "intsto", "inv", "isa",
			"isanimpol", "isbool", "ischar", "isclosedshape", "iscloseto",
			"isempty", "iseven", "isfun", "isfunvect", "isge", "isgt", "isint",
			"isintneg", "isinto", "isintpos", "isle", "islt", "ismat",
			"ismatof", "isnat", "isnull", "isnum", "isnumneg", "isnumpos",
			"isodd", "isometric", "isorthoshape", "ispair", "ispoint",
			"ispointseq", "ispol", "ispoldim", "ispolytope", "ispurepol",
			"isreal", "isrealneg", "isrealpos", "isrealvect", "isseq",
			"isseqof", "isshape", "issimplex", "issqrmat", "isstring",
			"isvect", "iszero", "jacobian", "join", "joints", "k", "last",
			"le", "left", "leftcavalier", "len", "less", "lesseq", "lex",
			"lift", "light", "linecolor", "linesize", "list", "ln", "load",
			"loadlib", "loop", "lt", "lxmy", "magenta", "map", "mapshapes",
			"markersize", "mat", "matdotprod", "material", "mathom", "max",
			"mean", "meanpoint", "med", "merge", "mesh", "min", "minkowski",
			"mirror", "mixedprod", "mk", "mkframe", "mkpol", "mkvector",
			"mkversork", "mod", "model", "move", "mul", "multextrude", "mxby",
			"mxmy", "mxty", "myfont", "n", "nat2string", "neq", "ngon",
			"norm2", "normalmap", "not", "nu_grid", "nubspline",
			"nubsplineknots", "nurbspline", "nurbsplineknots", "octahedron",
			"offset", "onepoint", "open", "optimize", "or", "orange", "ord",
			"ortho", "orthoproject", "orthox", "orthoy", "orthoz", "outarcs",
			"outerloop", "outerwarp", "pairdiff", "parallel", "pascaltriangle",
			"pdiff", "pdifference", "permutahedron", "permutations",
			"perspective", "perspective", "pi", "pivotop", "plane",
			"planemapping", "pmap", "points2shape", "polar", "polyline",
			"polymarker", "polypoint", "power", "powerset", "presort",
			"principalnormal", "print", "prism", "profileprodsurface",
			"progressivesum", "project", "projection", "purple", "pyramid",
			"q", "quadarray", "quadmesh", "quote", "r", "raise", "range",
			"rationalbezier", "rationalblend", "rationalbspline",
			"rationalize", "red", "rev", "reverse", "rgbacolor", "right",
			"rightcavalier", "ring", "rn", "rotatedtext", "rotationalsurface",
			"rotn", "rtail", "ruledsurface", "rxmy", "s", "save",
			"scalarmatprod", "scalarvectprod", "schlegel2d", "schlegel3d",
			"sdifference", "sdifferencepr", "segment", "sel", "setand",
			"setdiff", "setfontcolor", "setor", "setxor", "sex", "shape_0",
			"shape_1", "shape2points", "shape2pol", "shapeclosed", "shapecomb",
			"shapediff", "shapedist", "shapeinbetweening", "shapeinf",
			"shapejoin", "shapelen", "shapenorm", "shapenormal", "shapeprod",
			"shaperot", "shapesum", "shapesup", "shapezero", "shift",
			"showprop", "sign", "signal", "simplex", "simplexpile", "sin",
			"sinh", "size", "skeleton", "skew", "smaller", "smallest",
			"solidifier", "solidify", "sort", "sphere", "spline",
			"splinesampling", "splitcells", "splitpols", "sqr", "sqrt", "star",
			"string", "stringtokens", "struct", "sub", "svg", "sweep", "t",
			"tail", "tan", "tangent", "tanh", "tensorprodsurface ",
			"tetrahedron", "text", "texture", "textwithattributes ",
			"thinsolid", "threepoints", "time", "tmax", "tmin", "top", "torus",
			"torusmap", "trace", "trans", "tree", "trianglefan",
			"trianglestripe", "trimetric", "true", "truncone", "tt", "tube",
			"twopoints", "uk", "ukpol", "ukpolf", "union", "unionpr",
			"unitvect", "unprune", "up", "vect2dtoangle", "vect2mat",
			"vectdiff", "vectnorm", "vectprod", "vectsum", "view", "viewmodel",
			"viewmodel", "vrml", "warp", "warp", "where", "white", "with ",
			"xcavalier", "xor", "xquadarray", "xx", "xx", "ycavalier", "yellow"};

	/**
	 * Default constructor which creates all rules for scanning PLaSM code.
	 * Each rule define a Token object used to classify different type of
	 * code: Operators, Keywords, Digits, Strings, Comments and Default text.
	 * 
	 * @param colorProvider the color provider for all different PLaSM 
	 * syntax highlighting colors
	 */
	public PlasmCodeScanner(PlasmColorProvider colorProvider) {

		/*
		 * Token types: Operators, Keywords, Digits, Strings (and Characters),
		 * Comments, Default
		 */
		IToken operator = new Token(new TextAttribute(colorProvider
				.getColor(PlasmColorProvider.OPERATOR)));
		IToken keyword = new Token(new TextAttribute(colorProvider
				.getColor(PlasmColorProvider.KEYWORD)));
		IToken digit = new Token(new TextAttribute(colorProvider
				.getColor(PlasmColorProvider.DIGIT)));
		IToken string = new Token(new TextAttribute(colorProvider
				.getColor(PlasmColorProvider.STRING)));
		IToken comment = new Token(new TextAttribute(colorProvider
				.getColor(PlasmColorProvider.COMMENT)));
		IToken other = new Token(new TextAttribute(colorProvider
				.getColor(PlasmColorProvider.DEFAULT)));

		List rules = new ArrayList();

		/* Normal Code */
		PlasmWordRule wordRule = new PlasmWordRule(new PlasmWordDetector(),
				other);

		/* Normal Keywords */
		for (int i = 0; i < keywords.length; i++)
			wordRule.addWord(keywords[i], keyword);

		rules.add(wordRule);

		/* Single Operators */
		rules.add(new WordRule(new PlasmOperatorDetector(), operator));

		/* Rule for numerical values */
		rules.add(new NumberRule(digit));

		/* Rules for Strings and Character */
		rules.add(new SingleLineRule("\"", "\"", string));
		rules.add(new SingleLineRule("'", "'", string));

		/* Rules for Comments */
		rules.add(new MultiLineRule("%", "%", comment));

		// Add generic whitespace rule.
		rules.add(new WhitespaceRule(new PlasmWhitespaceDetector()));

		IRule[] result = new IRule[rules.size()];
		rules.toArray(result);
		setRules(result);
	}
}